"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const typescript_1 = require("typescript");
function plugin(program, pluginOptions = {}) {
    const specialExtensions = pluginOptions.specialExtensions || ['cjs', 'mjs', 'js', ''];
    let dependencies;
    let packageFound = false;
    if (pluginOptions['package.json']) {
        const path = path_1.resolve(program.getCurrentDirectory(), pluginOptions['package.json']);
        if (!fs_1.existsSync(path)) {
            throw new Error('package.json does not exists at ' + path);
        }
        packageFound = true;
        dependencies = createDeps(path) || {};
    }
    return (transformationContext) => (node) => {
        if (!packageFound) {
            packageFound = true;
            const path = findUp(path_1.dirname(node.fileName), 'package.json');
            if (path) {
                dependencies = createDeps(path) || {};
            }
        }
        if (!dependencies) {
            dependencies = {};
        }
        console.log('need wrap!!!', node.fileName);
        return typescript_1.visitEachChild(node, tokenWalk, transformationContext);
        function tokenWalk(node) {
            if (!isModuleNeedWrap(node, specialExtensions, dependencies)) {
                return node;
            }
            console.log(node.kind);
            return node;
        }
    };
}
exports.default = plugin;
function isModuleNeedWrap(node, specialExtensions, dependencies) {
    if (!typescript_1.isImportDeclaration(node) && !typescript_1.isExportDeclaration(node))
        return false;
    if (node.moduleSpecifier === undefined)
        return false;
    // only when module specifier is valid
    if (!typescript_1.isStringLiteral(node.moduleSpecifier))
        return false;
    const importPath = node.moduleSpecifier.text;
    // only when path is relative
    if (!importPath.startsWith('./') && !importPath.startsWith('../'))
        return false;
    // only when path does not have some special extension
    if (specialExtensions.includes(path_1.extname(importPath).replace(/^\./, '')))
        return false;
    // only when dependency defined in package.json
    const moduleName = importPath.split('/', importPath.startsWith('@') ? 2 : 1).join('/');
    if (!dependencies.hasOwnProperty(moduleName))
        return false;
    if (!dependencies[moduleName])
        dependencies[moduleName] = loadType(moduleName);
    // only if package.json found for target file
    if (dependencies[moduleName] === undefined)
        return false;
    return dependencies[moduleName] === 'commonjs';
}
function findUp(p, f) {
    const path = path_1.resolve(p, f);
    if (fs_1.existsSync(path)) {
        return path;
    }
    else if (p === '/') {
        return null;
    }
    else {
        return findUp(path_1.dirname(p), f);
    }
}
function loadType(moduleName) {
    const path = findUp(moduleName, path_1.join(moduleName, 'package.json'));
    if (path) {
        const pkg = JSON.parse(fs_1.readFileSync(path, 'utf-8'));
        return pkg.type || 'commonjs';
    }
    else {
        return undefined;
    }
}
function createDeps(path) {
    const text = fs_1.readFileSync(path, 'utf-8');
    const pkg = JSON.parse(text);
    if (!pkg.dependencies)
        return null;
    const deps = {};
    for (const i of Object.keys(pkg.dependencies)) {
        deps[i] = '';
    }
    return deps;
}
//# sourceMappingURL=register.js.map